module Hermes
  module Api
    module Dmo
      class StatusesController < Hermes::Api::Dmo::ApplicationController

       # before_action :set_instance, only: [:update, :destroy, :show]
        MAX_PICTURE_NUMBER = 5
        MUTATE_PARAMS = %i[
          status_label
          motive
          title
          object_id
          performer_id
        ]

        VALID_OPERATORS = %w[
          $in
          $nin
          $ne
          $regex
        ]

        ACCEPTABLE_PARAMS = %w[
          ids
          reference_model_name
          status_label
          motive
          title
          object_id
          performer_id
          closed_by
          closed_at
          closing
          applicant
          housing_request
          housing_offer
          housing_application
          entity
          lessor
        ]

        REGEX_PARAMS = %w[
          motive
          title
        ]
        
        def mutate_params
          MUTATE_PARAMS
        end 

        def current_user_is_admin
          !(current_user_roles & ['Administrator', 'superadministrator']).empty?
        end 


        def index
          return render_json_error(:unauthorized, :not_admin) unless current_user_is_admin
          
          acceptable_params *ACCEPTABLE_PARAMS
          regex_params *REGEX_PARAMS
          valid_operators *VALID_OPERATORS

          paginate(
            index_aggregation(no_acl: true),
            each_serializer: @serializer,
            **@options
          )
        end

        def show
          return render_json_error(:unauthorized, :not_admin) unless current_user_is_admin
          super
        end 

        def applications_history
          return render_json_error(:unauthorized, :not_demandeur) unless current_user_roles.include?('demandeur')

          matching = {
            reference_model_name: 'Hermes::HousingApplication',
            'housing_application.user_id' => @user.id
          }
          if params[:start_date].present? || params[:end_date].present?
            date_query = {}.tap do |h|
              h[:$gte] = Time.parse(params[:start_date]).beginning_of_day if params[:start_date].present?
              h[:$lte] = Time.parse(params[:end_date]).end_of_day if params[:end_date].present?
            end
            matching.merge!(updated_at: date_query)
          end

          aggregation = @model.collection.aggregate([
            { '$lookup' => { from: 'hermes_housing_applications', localField: 'housing_application_id', foreignField: '_id', as: 'housing_application' }},
            { '$match' => matching }
          ])

          paginate(
            aggregation,
            each_serializer: @serializer,
          )
        end
      end
    end
  end
end