module NotificationsCenter
  class HousingRequest < Base
    KEYS = NOTIFICATION_KEYS[:housing_request]
    def initialize(*arg)
      super
      @request = @instance.notifiable
      @number = @request.registration_number
    end

    def title
      case @key
        when KEYS[:delegation_signed_required]
          ''
        when KEYS[:entity_validated]
          "#{NAME} - Information sur votre demande"
        when KEYS[:custom]
          @instance.title
        when KEYS[:documents_consultation_ko]
          "#{NAME} - Action à effectuer sur votre espace"
        when KEYS[:documents_application_ko]
          "#{NAME} - Action à effectuer sur votre espace"
        when KEYS[:documents_application_missing]
          "#{NAME} - Action à effectuer sur votre espace"
        when KEYS[:status_change]
          "#{NAME} - Information sur votre demande"
        when KEYS[:request_incompleted]
          "#{NAME} - Action à effectuer sur votre espace"
        when KEYS[:documents_missing]
          "#{NAME} - Action à effectuer sur votre espace"
        when KEYS[:application_delay_exceeded]
          "#{NAME} - Action à effectuer sur votre espace"
        when KEYS[:request_completed], KEYS[:al_edit], KEYS[:applicant_edit]
          # just a front notif
          ""
        else
          ''
      end
    end

    def body
      case @key
        when KEYS[:delegation_signed_required]
          ''
        when KEYS[:custom]
          @instance.body
        when KEYS[:entity_validated]
          "
          Votre employeur a validé votre demande de logement.\n
          Avec cette validation complémentaire, nous vous informons que vous allez pouvoir accéder à davantage d'offres de logement. Nous vous invitons à vous rendre sur votre espace client pour consulter ces offres.
          "
        when KEYS[:documents_consultation_ko]
          html_doc_list = @instance.datas[:html_doc_list]
          "
            Après étude de vos pièces justificatives, nous ne pouvons valider votre demande.\n
            Merci de nous fournir les pièces justificatives suivantes :\n
            #{html_doc_list}
            <br/>
            Lorsque ces informations seront validées, vous pourrez accéder aux offres de logement et déposer votre candidature sur une offre de logement.
          "
        when KEYS[:documents_application_ko]
          html_doc_list = @instance.datas[:html_doc_list]
          "
            Après étude de votre dossier, nous ne pouvons pas prendre en compte votre candidature.\n
            Merci de nous fournir les pièces justificatives suivantes :\n
            #{html_doc_list}
            <br/>
            Lorsque ces informations seront validées votre candidature pourra être prise en compte. Merci de déposer ces pièces le plus rapidement possible. 
          "
        when KEYS[:documents_application_missing]
          html_doc_list = @instance.datas[:html_doc_list]
          "
            Après étude de vos pièces justificatives nous avons besoin de pièce complémentaire pour transmettre votre candidature au bailleur.\n
            Merci de nous fournir les pièces justificatives suivantes :\n
            #{html_doc_list}
            <br/>
            Lorsque ces informations seront validées votre candidature pourra être prise en compte. Merci de déposer ces pièces le plus rapidement possible si ces modifications ne sont pas réalisées sous 48h (jours ouvrés), votre candidature ne pourra pas être transmise au bailleur.
          "
        when KEYS[:request_incompleted]
          @instance.destroy # destroy notification (just send email)
          "
            Vous avez débuté la création d'une demande de logement sur #{@NAME}.\n
            Depuis le #{@request.created_at.strftime('%d/%m/%Y')}, vous n’avez pas finalisé l'enregistrement de vos informations. Votre demande n'est donc pas active.\n
            Nous vous invitons à finaliser votre demande et à déposer les pièces obligatoires au plus vite afin d'obtenir votre N° unique d'enregistrement\n
            <br/>
            Rappel: Pour pouvoir accéder aux offres de logement et déposer votre candidature vous devez disposer d'un N° unique.\n
            Si vous ne souhaitez plus faire de demande nous vous invitons à annuler votre saisie en cours sur votre espace, sans cela vous serez à nouveau relancé par mail.\n
          "
        when KEYS[:documents_missing]
          @instance.destroy # destroy notification (just send email)
          "
            Vous avez récemment modifié votre situation. Afin de valider cette modification et vous donner accès aux offres adaptées à celle-ci, veuillez déposer les pièces justificatives demandées sur votre espace. 
            Si cette modification n'est plus valable nous vous invitons à l'annuler, sans cela vous serez relancé par mail.
          "
        when KEYS[:application_delay_exceeded]
          "
            Après étude de votre candidature sur le logement N°#{@number} nous vous informons que votre dossier n'a pas été retenu.\n
            Malgré notre relance par mail, vous n'avez pas déposé les pièces justificatives nous permettant de transmettre votre dossier complet au bailleur.\n
            Nous vous invitons dès maintenant à vous rendre de nouveau sur votre espace, afin de postuler sur une autre offre de logement et mettre à jour les pièces justificatives demandées précédemment (plus d'informations dans votre espace rubrique 'Ma demande').\n
          "
        when KEYS[:status_change]
          "
            Changement de status
          "
        when KEYS[:request_completed], KEYS[:al_edit], KEYS[:applicant_edit]
          # just a front notif
          ""
        else
          ''
      end
    end

    def actions
      case @key
        when KEYS[:custom]
          {
            title: 'Voir les offres',
            link: '#connexion-demandeur'
          }
        when KEYS[:delegation_signed_required]
          {
            title: 'Voir les offres',
            link: '#connexion-demandeur'
          }
        when KEYS[:entity_validated]
          {
            title: 'Consulter les offres',
            link: '#connexion-demandeur'
          }
        when KEYS[:documents_consultation_ko]
          {
            title: 'Déposer mes pièces',
            link: '#connexion-demandeur'
          }
        when KEYS[:documents_application_ko]
          {
            title: 'Déposer mes pièces',
            link: '#connexion-demandeur'
          }
        when KEYS[:documents_application_missing]
          {
            title: 'Déposer mes pièces',
            link: '#connexion-demandeur'
          }
        when KEYS[:request_incompleted]
          {
            title: 'Compléter ma demande',
            link: '#connexion-demandeur'
          }
        when KEYS[:documents_missing]
          {
            title: 'Compléter ma demande',
            link: '#connexion-demandeur'
          }
        when KEYS[:application_delay_exceeded]
          {
            title: 'Voir les offres',
            link: '#connexion-demandeur'
          }
        when KEYS[:status_change]
          {
            title: 'Voir les offres',
            link: '#connexion-demandeur'
          }
        when KEYS[:request_completed], KEYS[:al_edit], KEYS[:applicant_edit]
          # just a front notif
          {
            title: '',
            link: ''
          }
        else
          ''
      end
    end
  end
end