# frozen_string_literal: true

module Hermes
  class HousingApplicationLightSerializer < Hermes::ApplicationSerializer
    type 'housing_applications'

    attributes %i[
      applicant_id
      housing_request_id
      housing_offer_id
      effort_rate
      profile_score
      status
      visit_status
      visit_conducted_date
      eligible_position
      applicated_position
      rank
      als_id
      date_event
      is_active
      request_als_id
      unique_numbers
      main_applicant_first_name
      main_applicant_last_name
      cotations
      business_cotation
      requester_firstname
      requester_lastname
      requester_email
      requester_birth_date
      applicant_used_for_siret_firstname
      applicant_used_for_siret_lastname
      applicant_used_for_siret_email
      applicant_used_for_siret_birth_date
    ]
    
    REQUEST_ATTRIBUTES = %w[
      seeked_localisation_list
      actual_housing_details
    ]

    REQUEST_ATTRIBUTES.each do |att|
      attributes att.to_sym
      define_method(:"#{att}") do
        object&.housing_request[att]
      end
    end 

    def main_applicant_first_name
      object.housing_request.main_applicant.first_name
    end
    
    def main_applicant_last_name
      object.housing_request.main_applicant.last_name
    end
    
    def request_als_id
      object.housing_request.als_id
    end
    
    def unique_numbers
      object.housing_request&.nuds&.pluck(:unique_number)
    end

    def applicant_id
      object.applicant_id&.to_s
    end
    
    def housing_request_id
      object.housing_request_id&.to_s
    end
    
    def housing_offer_id
      object.housing_offer_id&.to_s
    end
  end
end
