require 'rails_helper'

RSpec.describe Hermes::UpdateSeniorityPointJob do
  subject { Hermes::UpdateSeniorityPointJob }
  let(:redis_key) { 'update_seniority_point_job' }
  let!(:applicant) {create(:applicant)}
  let!(:hr) {create(:housing_request, main_applicant: applicant, is_active: true)}
  let!(:nud) {create(:nud, unique_number: '1', housing_request: hr)}

  before(:each) do
    $redis.del(redis_key)
    hr.validate_all_required_document
  end

  it 'should add job at hr creation' do
    expect( enqueued_jobs.select {|job| job[:job] == UpdateSeniorityPointJob}.length).to eq(1)
    drain_all_jobs
  end

  it 'should launch an other one' do
    Timecop.freeze(hr.calculate_next_seniority_updated_date)
    first_job = enqueued_jobs.select {|job| job[:job] == UpdateSeniorityPointJob}.first
    first_job[:job].perform_now(*first_job[:args])
    second_job = enqueued_jobs.select {|job| job[:job] == UpdateSeniorityPointJob}.last
    
    expect(hr.reload.seniority_points).to eq(1)
    expect(enqueued_jobs.select {|job| job[:job] == UpdateSeniorityPointJob}.length).to eq(2)
    expect( first_job[:at]).to_not eq(second_job[:at])

    Timecop.freeze( hr.reload.calculate_next_seniority_updated_date)
    second_job[:job].perform_now(*second_job[:args])
    third_job = enqueued_jobs.select {|job| job[:job] == UpdateSeniorityPointJob}.last
    
    expect( hr.reload.seniority_points).to eq(2)
    expect(enqueued_jobs.select {|job| job[:job] == UpdateSeniorityPointJob}.length).to eq(3)
    expect( third_job[:at]).to_not eq(second_job[:at])
    Timecop.return
  end 

end
